#!/usr/bin/python3

import os
import sys
import shutil
import fnmatch
import re
import argparse


def pause():
    # Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")


def print_format(text, value):
    # Substitute {0} in text with value.
    print(text.format(value))


def get_files(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files, value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches


def get_folders(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for dirs in fnmatch.filter(files, value):
            matches.append(dirs)
    # Sort folder alphabetically
    matches.sort()

    return matches


def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)


def list_files(path, value):
    matches = get_files(path, value)
    print_list(matches)


def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches) / divisor)):
            j = divisor * (i * 1)
            result = ''
            for j in range(j, j + (divisor)):
                result = result + matches[j] + '\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor) * divisor)

    result = ''
    for m in range(l, len(matches)):
        result = result + matches[m] + '\t'
    print(result)


def mov_files(path, dpath, value):
    # Create destination folder if it doesn't exist.
    ensure_dir(dpath)

    # list_files(path,value)
    # confirm=input("Enter Y to proceed: ")
    confirm = 'y'

    if confirm.lower() == 'y':
        try:
            matches = get_files(path, value)
            for i in range(len(matches)):
                source = os.path.join(path, matches[i])
                if os.path.isfile(source):
                    print("Moving: ", matches[i])
                    dest = os.path.join(dpath, matches[i])
                    shutil.move(source, dest)
            print("Move complete.")
        except:
            print("Problem moving files.")

    # pause()


def get_prefixes(files):
    matches = []
    t = []
    r = re.compile("([a-zA-Z]+[0-9]+-)|([a-zA-Z]+)")

    for file in files:
        m = r.match(file)
        if m:
            if (m.group(1)):
                t.append(m.group(1))
            else:
                t.append(m.group(2))

    matches = list(set(t))
    matches.sort(reverse=True)

    # print_list(matches)
    # print()
    # pause()

    return matches


def main():
    """Move fanfic chapters into story folders."""
    parser = argparse.ArgumentParser(
        description=('Utility to move fanfic chapters into story folders.'))

    parser.add_argument('folder', help='fanfic folder to process')

    args = parser.parse_args()

    source = args.folder

    if not os.path.exists(source):
        sys.stderr.write("ERROR: Folder %r was not found!" % (source,))
        return 1

    files = get_files(source, "*.txt")
    prefs = get_prefixes(files)

    for p in prefs:
        d = p.split("-")[0]
        dest = os.path.join(source, d)
        print_format("Moving files into folder: {0}", d)
        mov_files(source, dest, p + "*.txt")


if __name__ == "__main__":
    sys.exit(main())



import sys
import datetime
from pony.orm import Database, DatabaseError, Required, PrimaryKey, Optional
from pony.orm import sql_debug

db = Database()


def init_db(filename):
    bind_db(filename)
    db.generate_mapping(create_tables=True)


def bind_db(filename):

    try:
        db.bind(provider='sqlite', filename=filename, create_db=True)
    except DatabaseError as err:
        print("Error loading database!")
        sys.exit(1)


#############################################################
class Category(db.Entity):
    """
    Pony ORM model of the Category table
    """
    Name = Required(str)
    Id = PrimaryKey(int, auto=True)


#############################################################
class Fanfic(db.Entity):
    """
    Pony ORM model of the Fanfic table
    """
    Title = Required(str)
    Author = Required(str)
    Folder = Required(str)
    Chapter = Optional(str)
    Count = Optional(int)
    Matchup = Optional(str)
    Crossover = Optional(str)
    Description = Optional(str)
    Internet = Optional(str)
    StoryId = Optional(str)
    Abandoned = Optional(bool, default=0)
    Complete = Optional(bool, default=0)
    Publish_Date = Optional(datetime.date)
    Update_Date = Optional(datetime.date)
    Last_Checked = Optional(datetime.date)
    Id = PrimaryKey(int, auto=True)
    Category_Id = Required(int)


class DataBaseLogic:

    def __init__(self, db_filename):

        init_db(db_filename)


def main():

    bind_db('FanfictionDB.db')

    # turn on debug mode
    sql_debug(True)

    # map the models to the database
    # and create the tables, if they don't exist
    db.generate_mapping(create_tables=True)


if __name__ == '__main__':
    main()



#!/usr/bin/env python3

import os
import sys
import fnmatch
import subprocess
import argparse
from datetime import date
from collections import defaultdict
from configparser import SafeConfigParser
from fanfic_scraper import current_fanfic
from pony.orm import db_session, select, count
from fanfic_scraper.db_pony import DataBaseLogic, Category, Fanfic
from fanfic_scraper import cui
import textwrap
from time import sleep


basePath = '/home/ubuntu/OneDrive/'
arcRoot = 'FF_Archive'
db = None
db_name = 'FanfictionDB.db'
db_folder = 'Read'
dl_folder = 'htm'


def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)


def get_config():
    # create config dir if it doesn't exist and return path to config.
    home = os.path.expanduser("~")
    cfg = os.path.join(home, ".ff")
    ensure_dir(cfg)

    return os.path.join(cfg, "config.ini")


def config_load():
    global basePath
    global arcRoot
    global db_folder
    global db_name
    global dl_folder
    t_path = ''
    t_root = ''
    t_dbname = ''
    t_dbfolder = ''
    t_dlfolder = ''
    config = SafeConfigParser()

    cfg = get_config()

    if os.path.isfile(cfg):
        config.read(cfg)

        try:
            t_path = config.get('path', 'path')
            t_root = config.get('path', 'rootfolder')
            t_dbfolder = config.get('path', 'dbfolder')
            t_dbname = config.get('path', 'dbname')
            t_dlfolder = config.get('path', 'downloadfolder')
        except:
            i = 0

        if t_path:
            basePath = t_path
        if t_root:
            arcRoot = t_root
        if t_dbfolder:
            db_folder = t_dbfolder
        if t_dbname:
            db_name = t_dbname
        if t_dlfolder:
            dl_folder = t_dlfolder


def config_save():
    cfg = get_config()

    config = SafeConfigParser()

    config.read(cfg)

    if not config.has_section('path'):
        config.add_section('path')

    config.set('path', 'path', basePath)
    config.set('path', 'rootfolder', arcRoot)
    config.set('path', 'dbfolder', db_folder)
    config.set('path', 'dbname', db_name)
    config.set('path', 'downloadfolder', dl_folder)

    with open(cfg, 'w') as f:
        config.write(f)


def IsNotStale(update_date, last_checked):
    wait = 0
    ret = False

    if not last_checked:
        ret = True

    if not ret:
        if not update_date:
            ret = True

    if not ret:

        d0 = update_date
        d1 = last_checked
        delta = d1 - d0

        if delta.days > 360:
            wait = 180
        if delta.days > 180:
            wait = 90
        if delta.days > 90:
            wait = 60
        if delta.days > 60:
            wait = 30
        if delta.days > 30:
            wait = 7
        if delta.days < 30:
            wait = 0

        d0 = last_checked
        d1 = date.today()
        delta = d1 - d0

        if delta.days > wait and wait > 0:
            ret = False
        else:
            ret = True

    return ret


def set_ffargs(location, folder):

    parser = argparse.ArgumentParser(description=('fanfic_scraper args'))

    parser.add_argument(
        "-f", "--folder", required=True,
        help="Name of folder to place fanfic in.")
    parser.add_argument(
        "-l", "--location", default=os.getcwd(),
        help="set download location")
    # parser.add_argument(
    #    "-c", "--chapters", default=False,
    #    help="Specify chapters to download separated by : (10:15).")
    parser.add_argument(
        "-ct", "--chapterthreads", default=1,
        help="Number of parallel chapters downloads.")
    parser.add_argument(
        "-wt", "--waittime", default=15,
        help="Wait time before retry if encountered with an error")
    parser.add_argument(
        "-rt", "--retries", default=10,
        help="Number of retries before giving up")

    args = parser.parse_args(['--location=' + location, '--folder=' + folder])

    return args


def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches) / divisor)):
            j = divisor * (i * 1)
            result = ''
            for j in range(j, j + (divisor)):
                result = result + matches[j] + '\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor) * divisor)

    result = ''
    for m in range(l, len(matches)):
        result = result + matches[m] + '\t'
    print(result)


def get_subfolders(path):

    subfolders = [f.name for f in os.scandir(path) if f.is_dir()]
    subfolders.sort()

    return subfolders


def get_files(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files, value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches


def convert_file(source, htm):

    txt = htm.split('.')[0] + '.txt'
    ifile = [os.path.abspath(os.path.join(source, htm))]
    ofile = os.path.abspath(os.path.join(source, txt))
    cmd = ['html2text', '--ignore-emphasis', '-b 80']
    msg = 'Converting file: {0} to text file: {1}'

    print(msg.format(htm, txt))

    with open(ofile, "w") as outfile:
        subprocess.call(cmd + ifile, stdout=outfile)

    for i in ifile:
        os.remove(i)


class FanficDB:

    def __init__(self):
        db_path = os.path.join(basePath, arcRoot, db_folder, db_name)
        dbhandle = DataBaseLogic(db_path)

    def add_story(self, folder, cat_id, url, info):

        with db_session:
            Fanfic(Title=info['Title'],
                   Author=info['Author'],
                   Folder=folder,
                   Count=0,
                   Description=info['Description'],
                   Internet=url,
                   StoryId=info['StoryId'],
                   Publish_Date=info['Publish_Date'],
                   Update_Date=info['Update_Date'],
                   Last_Checked=info['Publish_Date'],
                   Category_Id=cat_id)

    def add_cat(self, category):

        with db_session:
            Category(Name=category)

    def get_categories(self):
        d = defaultdict(list)

        with db_session:
            categories = select(c for c in Category
                                if c.Id > 0).order_by(Category.Name)

            for c in categories:
                for k, v in c.to_dict().items():
                    d[k].append(v)

        return d['Name']

    def get_cat_count(self, categoryName):

        with db_session:
            category = count(c for c in Category
                             if c.Name == categoryName
                             and c.Id > 0)

        return category

    def get_folder(self, cat_id, folderName):

        with db_session:
            folders = count(f.Folder for f in Fanfic
                            if f.Category_Id == cat_id
                            and f.Folder == folderName)

        return folders

    def get_cat_id(self, categoryName):

        with db_session:
            cat = Category.get(Name=categoryName)
            ret = cat.Id

        return ret

    def get_fanfics(self, cat_id):
        d = defaultdict(list)
        ret = []

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Abandoned == 0
                        and f.Complete == 0
                        and f.Category_Id == cat_id)
            ff = ff.order_by(Fanfic.Folder)

            for f in ff:
                d = defaultdict(list)
                for k, v in f.to_dict().items():
                    d[k].append(v)
                ret.append(d)

        return ret

    def update_chapter_count(self, fic_id, chapters):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Count = chapters

    def update_last_checked(self, fic_id, last):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Last_Checked = last

    def update_last_updated(self, fic_id, update):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Update_Date = update


def download_stories(args):

    # Ensure download folder exists prior to starting download.
    location = os.path.join(basePath, arcRoot, dl_folder)
    ensure_dir(location)

    cat = db.get_categories()

    for cat_name in cat:
        # print("Processing:", cat_name)
        download_by_category(cat_name)
        convert_by_category(cat_name)


def download_by_category(category):

    potential_keys = []

    cat_folder = category.replace(' ', '_')

    location = os.path.join(basePath, arcRoot, dl_folder, cat_folder)

    # Ensure category folder exists prior to starting download.
    ensure_dir(location)

    cat_id = db.get_cat_id(category)

    fics = db.get_fanfics(cat_id)

    fic_idx = 0
    fic_count = len(fics)
    msg = "Processing: {0} - {1} of {2}."

    for fic in fics:
        fic_idx = fic_idx + 1
        print(msg.format(category, fic_idx, fic_count), end='\r')
        title = fic['Title'][0]
        author = fic['Author'][0]
        folder = fic['Folder'][0]
        chapters = fic['Count'][0]
        update = fic['Update_Date'][0]
        last = fic['Last_Checked'][0]
        if fic['Internet'][0]:
            tmp = fic['Internet'][0]
            tmp2 = tmp.split('#')
            if len(tmp2) > 1:
                url = fic['Internet'][0].split('#')[1]
            else:
                url = tmp
        else:
            url = ''
        storyid = fic['StoryId'][0]
        fic_id = fic['Id'][0]

        date_check = IsNotStale(update, last)
        url_check = current_fanfic.check_url(url)

        if url_check is True and date_check is True:

            ffargs = set_ffargs(location, folder)

            next_chapter = chapters + 1

            # Initialize class so we can retrieve actual story url
            fanfic = current_fanfic.fanfic(url, ffargs)
            # Get fanfic url
            url = fanfic.get_story_url(storyid)
            # Initialize class with correct url.
            fanfic = current_fanfic.fanfic(url, ffargs)
            # Download list of chapters
            fanfic.get_chapters()
            # Get current chapter count from site
            cc = fanfic.chapter_count

            if cc >= next_chapter:
                if cc == next_chapter:
                    potential_keys = [next_chapter]
                else:
                    potential_keys = [
                        i * 1 for i in range(next_chapter, (cc + 1))]
                # Show story about to be updated.
                update = fanfic.get_update_date()
                print('', end='\n')
                print(title, '/', folder, '/', update)
                # Set chapters to download
                fanfic.set_download_chapters(potential_keys)
                # Download Story
                fanfic.download_fanfic()
                print('Downloaded fanfic: ', category, '/', title, '/', folder)
                db.update_chapter_count(fic_id, cc)
                db.update_last_updated(fic_id, update)
                db.update_last_checked(fic_id, date.today())
            else:
                db.update_last_checked(fic_id, date.today())

            sleep(4)
        else:
            sleep(0.2)

    print('', end='\n')


def convert_by_category(category):

    cat_folder = category.replace(' ', '_')
    location = os.path.join(basePath, arcRoot, dl_folder, cat_folder)

    folders = get_subfolders(location)

    for folder in folders:
        target = os.path.realpath(os.path.join(location, folder))
        files = get_files(target, "*.htm")

        for htm in files:
            convert_file(target, htm)


def add_story(category, cat_id, folder):
    _ = os.system("clear")
    url = input("Enter Story URL: ")
    url_check = check_url(url)
    if url_check:
        ffargs = set_ffargs('/tmp', folder)
        fanfic = current_fanfic.fanfic(url, ffargs)
        info = fanfic.story_info()
        print("ID: {0}".format(info['StoryId']))
        print("Title: {0}".format(info['Title']))
        print("Author: {0}".format(info['Author']))
        print("Summary: ", textwrap.fill(info['Description']))
        print("Published: {0}".format(info['Publish_Date']))
        print("Updated: {0}".format(info['Update_Date']))
        print("Chapter Count: {0}".format(info['Count']))
        cui.pause()
        confirm = cui.menu_yesno("Proceed?")

        if confirm == "Yes":
            db.add_story(folder, cat_id, url, info)
            print("Story added to database.")
            cui.pause()

    else:
        print("Site not supported!!!")
        cui.pause()


def add_cat():
    _ = os.system("clear")
    category = input("Enter Category Name: ")
    cat_count = db.get_cat_count(category)

    if cat_count > 0:
        print("Category: {0} already exists!".format(category))
        category = "None"
        cui.pause()
    else:
        db.add_cat(category)

    return category


def config_menu():
    category = "None"
    folder = "None"

    while True:
        menu = []
        menu.append(cui.get_entry("Category: {0}", category))
        menu.append(cui.get_entry("Folder: {0}", folder))
        menu.append("Add Category")
        menu.append("Add Story")
        menu.append("Exit")

        _ = os.system("clear")

        print("Fanfic Downloader - Config")
        print("")
        ret = cui.submenu(menu, "Enter Selection")

        if "Category:" in ret:
            cat = db.get_categories()
            _ = os.system("clear")
            category = cui.submenu(cat, "Choose Category: ")
            cat_id = db.get_cat_id(category)
        if "Folder:" in ret:
            if category != "None":
                folder = input("Enter Folder Name: ")
                folder_count = db.get_folder(cat_id, folder)
                if folder_count > 0:
                    print("Folder: {0} already exists!".format(folder))
                    folder = "None"
                    cui.pause()
            else:
                print("Please select category first!")
                cui.pause()
        if ret == "Add Category":
            category = add_cat()
        if ret == "Add Story":
            if category != "None" and folder != "None":
                add_story(category, cat_id, folder)
                folder = "None"
            else:
                print("Please ensure Category and Folder are not set to None!")
                cui.pause()
        if ret == "Exit":
            sys.exit(0)


def main():
    global db
    parser = argparse.ArgumentParser(
        description=(
            'Automates fanfic downloads latest chapters using fanfic_scraper. '
            '(Currently works with fanfiction.net)'))
    parser.add_argument(
        "-c", '--config', required=False, action='store_true',
        help="Configures Fanfic Downloader.")

    config_load()
    db = FanficDB()

    args = parser.parse_args()

    if args.config:
        config_menu()
    else:
        download_stories(args)


if __name__ == "__main__":
    sys.exit(main())



"""Extractor for fanfiction.net."""

from fanfic_scraper.base_fanfic import BaseFanfic, BaseChapter
from urllib.parse import urlparse, urljoin
import bs4 as bsoup
from collections import defaultdict
import os
from datetime import datetime


def chapter_nav(tag):
    test = (tag.name == 'select')
    test = (test and 'chap_select' in tag['id'])

    return test


class FanfictionNetFanfic(BaseFanfic):

    def get_story_url(self, storyid):
        return 'https://www.fanfiction.net/s/' + storyid

    def extract_chapters(self):
        """Extract chapters function (backbone)."""
        fanfic_name = self.name
        url = self.url
        self.title = url.split('/')[-1]
        urlscheme = urlparse(url)

        # set story_id from url
        self.fanfic_id = urlscheme.path.split('/')[2]

        # Get chapters
        r = self.send_request(url)
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        chapters = defaultdict(FanfictionNetChapter)
        chapter_list = soup.find_all(chapter_nav)

        if chapter_list:
            chapter_list = chapter_list[0]

        if chapter_list:
            for link in chapter_list:
                chapter_num = int(link.get('value'))
                chapter_link = urljoin(
                    urlscheme.scheme + "://" + urlscheme.netloc,
                    "s/" + self.fanfic_id + "/" + str(chapter_num))
                chapters[chapter_num] = FanfictionNetChapter(
                    self, chapter_num, chapter_link)
        else:
            chapter_num = 1
            chapter_link = urljoin(
                urlscheme.scheme + "://" + urlscheme.netloc,
                "s/" + self.fanfic_id + "/" + str(chapter_num))
            chapters[chapter_num] = FanfictionNetChapter(
                self, chapter_num, chapter_link)

        return chapters

    def get_update_date(self):
        r = self.send_request(self.url)
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'profile_top'}):
            span = div.find_all('span')[-2]
            ts = span.get('data-xutime')

            if ts is None:
                span = div.find_all('span')[-1]
                ts = span.get('data-xutime')

            update_date = datetime.fromtimestamp(float(ts))
            update_date = update_date.strftime('%Y-%m-%d')

            break

        return update_date


class FanfictionNetChapter(BaseChapter):
    """Base chapter class."""

    def get_fanfic_title(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'profile_top'}):
            title = div.find_all('b')[0].get_text()
            break

        return title

    def get_fanfic_author(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'profile_top'}):
            for link in div.find_all('a'):
                if '/u' in link.get('href'):
                    author = link.get_text()
                    break

        return author

    def get_fanfic_category(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'pre_story_links'}):
            for link in div.find_all('a'):
                category = link.get_text()
                break

        return category

    def get_fanfic_description(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'profile_top'}):
            description = div.find_all('div')[1].get_text()
            break

        return description

    def get_update_date(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'profile_top'}):
            span = div.find_all('span')[-2]
            ts = span.get('data-xutime')

            if ts is None:
                span = div.find_all('span')[-1]
                ts = span.get('data-xutime')

            update_date = datetime.fromtimestamp(float(ts))
            update_date = update_date.strftime('%Y-%m-%d')

            break

        return update_date

    def get_publish_date(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'profile_top'}):
            span = div.find_all('span')[-1]
            ts = span.get('data-xutime')
            publish_date = datetime.fromtimestamp(float(ts))
            publish_date = publish_date.strftime('%Y-%m-%d')

            break

        return publish_date

    def get_chapter_title(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        chapter_list = soup.find_all(chapter_nav)

        if chapter_list:
            chapter_list = chapter_list[0]

            for link in chapter_list:
                if int(link.get('value')) == self.chapter_num:
                    chapter_title = link.get_text()
                    break
        else:
            chapter_title = str(self.chapter_num)

        return chapter_title

    def get_chapter_count(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        chapter_list = soup.find_all(chapter_nav)

        if chapter_list:
            chapter_count = len(chapter_list[0])
        else:
            chapter_count = 1

        return chapter_count

    def get_chapter_html(self, r):
        soup = bsoup.BeautifulSoup(r.text, 'html5lib')

        story = soup.find_all('div', {'id': 'storytext'})[0]

        return str(story)

    def render_p(self, value):
        return '<p>' + value + '</p>'

    def story_info(self):
        r = self.send_request(self.chapter_url)

        title = self.get_fanfic_title(r)
        author = self.get_fanfic_author(r)
        category = self.get_fanfic_category(r)
        desc = self.get_fanfic_description(r)
        update_date = self.get_update_date(r)
        publish_date = self.get_publish_date(r)
        chapter_count = self.get_chapter_count(r)

        info = {}

        info['StoryId'] = self.fanfic_id
        info['Title'] = title
        info['Author'] = author
        info['Category'] = category
        info['Description'] = desc
        info['Publish_Date'] = publish_date
        info['Update_Date'] = update_date
        info['Count'] = chapter_count

        return info

    def download_chapter(self):

        filename = self.fanfic_name + '-%03d.htm' % (self.chapter_num)

        r = self.send_request(self.chapter_url)

        title = self.get_fanfic_title(r)
        author = self.get_fanfic_author(r)
        category = self.get_fanfic_category(r)
        desc = self.get_fanfic_description(r)
        chapter_title = self.get_chapter_title(r)
        update_date = self.get_update_date(r)
        publish_date = self.get_publish_date(r)
        chapter_count = self.get_chapter_count(r)
        story = self.get_chapter_html(r)

        # print(title)
        # print(author)
        # print(category)
        # print("Summary: ", textwrap.fill(desc))
        # print('Chapter '+chapter_title)
        # print('Published: '+publish_date)
        # print('Updated: '+update_date)
        # print(chapter_count)
        # print(story)

        target = os.path.join(self.fanfic_download_location, filename)

        if os.path.isfile(target):
            os.remove(target)

        f1 = open(target, "w")

        f1.write('<html>')
        f1.write('<body>')
        f1.write(self.render_p(title))
        f1.write(self.render_p(author))
        f1.write(self.render_p(category))
        f1.write(self.render_p('Summary: ' + desc))
        f1.write(self.render_p('Chapter ' + chapter_title))
        if self.chapter_num == 1:
            f1.write(self.render_p('Published: ' + publish_date))
        if self.chapter_num == chapter_count:
            f1.write(self.render_p('Updated: ' + update_date))
        f1.write(self.render_p('========='))
        f1.write(story)

        f1.flush()
        os.fsync(f1.fileno())
        f1.close



"""Extractor for hpfanficarchive.com."""

from fanfic_scraper.base_fanfic import BaseFanfic, BaseChapter
from urllib.parse import urlparse, urljoin, parse_qs
from bs4 import BeautifulSoup, Comment
from collections import defaultdict
import re
import os
from datetime import datetime


def chapter_nav(tag):
    test = (tag.name == 'select')
    test = (test and 'chap_select' in tag['id'])

    return test


class HPFanficArchive(BaseFanfic):

    def get_fanfic_title(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')

        for div in soup.find_all('div', {'id': 'pagetitle'}):
            ch_regex = re.compile(r'^viewstory.php\?sid=')
            title = div.find_all('a', href=ch_regex)[0]
            title = title.get_text()
            break

        return title

    def get_story_url(self, storyid):
        base_url = 'https://www.hpfanficarchive.com/stories/viewstory.php?sid='
        return base_url + storyid

    def extract_chapters(self):
        """Extract chapters function (backbone)."""
        fanfic_name = self.name

        url = self.url
        urlscheme = urlparse(url)

        # Set story_id from url
        self.fanfic_id = parse_qs(urlscheme.query,
                                  keep_blank_values=True)['sid'][0]

        # Get chapters
        r = self.send_request(url)
        soup = BeautifulSoup(r.text, 'html5lib')
        self.title = self.get_fanfic_title(r)
        chapters = defaultdict(HPFanficArchiveChapter)

        try:

            ch_regex = re.compile(r'^viewstory.php\?sid=')
            chapter_list = soup.find_all('a', href=ch_regex)

            for link in chapter_list:
                chapter = link.get('href')
                if 'chapter' in chapter:
                    chapter_link = urljoin(
                        urlscheme.scheme + "://" + urlscheme.netloc,
                        'stories/' + str(chapter))
                    ch_qs = parse_qs(urlparse(chapter_link).query)
                    chapter_num = ch_qs['chapter'][0]
                    chapter_num = int(chapter_num)
                    chapters[chapter_num] = HPFanficArchiveChapter(
                        self, chapter_num, chapter_link)

            return chapters

        except:
            return chapters

    def get_update_date(self):
        r = self.send_request(self.url)
        soup = BeautifulSoup(r.text, 'lxml')

        for c in soup.find_all(text=lambda text: isinstance(text, Comment)):
            if c in [' UPDATED START ']:
                update_date = c.next_element.strip()
                update_date = datetime.strptime(update_date, '%B %d, %Y')
                break

        return update_date


class HPFanficArchiveChapter(BaseChapter):
    """Base chapter class."""

    def get_fanfic_title(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')

        regex = re.compile(r'^viewstory.php\?sid=')

        for div in soup.find_all('div', {'id': 'pagetitle'}):
            title = div.find_all('a', href=regex)[0]
            title = title.get_text()
            break

        return title

    def get_fanfic_author(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')

        regex = re.compile(r'^viewuser.php\?uid=')

        for div in soup.find_all('div', {'id': 'pagetitle'}):
            author = div.find_all('a', href=regex)[0]
            author = author.get_text()
            break

        return author

    def get_fanfic_category(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')
        category = ''

        regex = re.compile(r'^browse.php\?type=categories')

        desc = soup.find_all('div', {'class': 'content'})[2]
        cat = desc.find_all('a', href=regex)
        cat2 = []
        for a in cat:
            cat2.append(a.get_text())
        s = ', '
        category = s.join(cat2)

        return category

    def get_fanfic_genre(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')
        category = ''

        regex = re.compile(r'type_id=1')

        desc = soup.find_all('div', {'class': 'content'})[2]
        cat = desc.find_all('a', href=regex)
        cat2 = []
        for a in cat:
            cat2.append(a.get_text())
        s = ', '
        category = s.join(cat2)

        return category

    def get_fanfic_description(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')

        desc = soup.find_all('div', {'class': 'content'})[2]
        para = desc.find_all('p')
        temp = []
        for p in para:
            temp.append(p.get_text())
        desc = "".join(temp)
        return desc

    def get_update_date(self, r):
        soup = BeautifulSoup(r.text, 'lxml')

        for c in soup.find_all(text=lambda text: isinstance(text, Comment)):
            if c in [' UPDATED START ']:
                update_date = c.next_element.strip()
                break

        return update_date

    def get_publish_date(self, r):
        soup = BeautifulSoup(r.text, 'lxml')

        for c in soup.find_all(text=lambda text: isinstance(text, Comment)):
            if c in [' PUBLISHED START ']:
                publish_date = c.next_element.strip()
                break

        return publish_date

    def get_chapter_title(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')

        chapters = soup.find_all('select', {'name': 'chapter'})[0]
        chapter_list = chapters.find_all('option')
        for option in chapter_list:
            if int(option.get('value')) == self.chapter_num:
                chapter_title = option.get_text()
                break

        return chapter_title

    def get_chapter_count(self, r):
        """Extract chapters function (backbone)."""

        soup = BeautifulSoup(r.text, 'html5lib')

        chapters = 0

        try:

            ch_regex = re.compile(r'^viewstory.php\?sid=')
            chapter_list = soup.find_all('a', href=ch_regex)

            for link in chapter_list:
                chapter = link.get('href')
                if 'chapter' in chapter:
                    chapters = chapters + 1

            return chapters

        except:
            return chapters

    def get_chapter_html(self, r):
        soup = BeautifulSoup(r.text, 'html5lib')

        story = soup.find_all('div', {'id': 'story'})[0]

        return str(story)

    def render_p(self, value):
        return '<p>' + value + '</p>'

    def story_info(self):

        r = self.send_request(self.fanfic_url)

        title = self.get_fanfic_title(r)
        author = self.get_fanfic_author(r)
        category = self.get_fanfic_category(r)
        genre = self.get_fanfic_genre(r)
        desc = self.get_fanfic_description(r)
        update_date = self.get_update_date(r)
        publish_date = self.get_publish_date(r)
        chapter_count = self.get_chapter_count(r)

        info = {}

        info['StoryId'] = self.fanfic_id
        info['Title'] = title
        info['Author'] = author
        info['Description'] = desc
        info['Publish_Date'] = publish_date
        info['Update_Date'] = update_date
        info['Count'] = chapter_count

        return info

    def download_chapter(self):

        filename = self.fanfic_name + '-%03d.htm' % (self.chapter_num)

        print(self.chapter_url)
        r = self.send_request(self.fanfic_url)

        title = self.get_fanfic_title(r)
        author = self.get_fanfic_author(r)
        category = self.get_fanfic_category(r)
        genre = self.get_fanfic_genre(r)
        desc = self.get_fanfic_description(r)
        update_date = self.get_update_date(r)
        publish_date = self.get_publish_date(r)
        chapter_count = self.get_chapter_count(r)

        r = self.send_request(self.chapter_url)

        chapter_title = self.get_chapter_title(r)
        story = self.get_chapter_html(r)

        # print(title)
        # print(author)
        # print('Categories: '+category)
        # print('Genres: '+genre)
        # print("Summary: ", textwrap.fill(desc))
        # print('Chapter '+chapter_title)
        # print('Published: '+publish_date)
        # print('Updated: '+update_date)
        # print(chapter_count)
        # print(story)

        target = os.path.join(self.fanfic_download_location, filename)

        if os.path.isfile(target):
            os.remove(target)

        f1 = open(target, "w")

        f1.write('<html>')
        f1.write('<body>')
        f1.write(self.render_p(title))
        f1.write(self.render_p(author))
        f1.write(self.render_p('Categories: ' + category))
        f1.write(self.render_p('Summary: ' + desc))
        f1.write(self.render_p('Chapter ' + chapter_title))
        if self.chapter_num == 1:
            f1.write(self.render_p('Published: ' + publish_date))
        if self.chapter_num == chapter_count:
            f1.write(self.render_p('Updated: ' + update_date))
        f1.write(self.render_p('========='))
        f1.write(story)

        f1.flush()
        os.fsync(f1.fileno())
        f1.close



#!/usr/bin/python3

import os
import sys
import shutil
import fnmatch
import subprocess
import textwrap
import json
import argparse
from datetime import date
from collections import defaultdict
from configparser import SafeConfigParser, Error
from fanfic_scraper import current_fanfic
from pony.orm import db_session, select, count
from fanfic_scraper.db_pony import DataBaseLogic, Category, Fanfic
from fanfic_scraper import cui


category = ''
folder = ''
dfolder = ''
sfolder = ''
tfile = ''
editor = 'vi'
sync_server = ''
sync_path = ''
sync_safe = True

# Initial Values
basePath = '/home/ubuntu/OneDrive/'
arcRoot = 'FF_Archive'
db = None
db_name = 'FanfictionDB.db'
db_folder = 'Read'


class FanficDB:

    def __init__(self):
        db_path = os.path.join(basePath, arcRoot, db_folder, db_name)
        dbhandle = DataBaseLogic(db_path)

    def get_cat_id(self, categoryName):

        with db_session:
            cat = Category.get(Name=categoryName)
            ret = cat.Id

        return ret

    def set_complete(self, fic_id):

        with db_session:
            fic = Fanfic.get(Id=fic_id)
            c = fic.Complete

            if c == 1:
                c = 0
            else:
                c = 1

            fic.Complete = c

    def set_abandoned(self, fic_id):

        with db_session:
            fic = Fanfic.get(Id=fic_id)
            a = fic.Abandoned

            if a == 1:
                a = 0
            else:
                a = 1

            fic.Abandoned = a

    def get_fic_id(self, cat_id, folder):

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Category_Id == cat_id
                        and f.Folder == folder)

            for f in ff:
                ret = f.Id

        return ret

    def get_fanfic(self, cat_id, folder):
        d = defaultdict(list)
        ret = []

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Category_Id == cat_id
                        and f.Folder == folder)

            for f in ff:
                d = defaultdict(list)
                for k, v in f.to_dict().items():
                    d[k].append(v)
                ret.append(d)

        return ret[0]
    
    def get_fanfic_byId(self, fic_id):
        d = defaultdict(list)
        ret = []

        with db_session:
            ff = select(f for f in Fanfic
                        if f.Id == fic_id)

            for f in ff:
                d = defaultdict(list)
                for k, v in f.to_dict().items():
                    d[k].append(v)
                ret.append(d)

        return ret[0]

    def update_chapter_count(self, fic_id, chapters):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Count = chapters

    def update_last_checked(self, fic_id, last):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Last_Checked = last

    def update_last_updated(self, fic_id, update):

        with db_session:
            ff = Fanfic.get(Id=fic_id)
            ff.Update_Date = update


def set_ffargs(location, folder):

    parser = argparse.ArgumentParser(description=('fanfic_scraper args'))

    parser.add_argument(
        "-f", "--folder", required=True,
        help="Name of folder to place fanfic in.")
    parser.add_argument(
        "-l", "--location", default=os.getcwd(),
        help="set download location")
    #parser.add_argument(
    #    "-c", "--chapters", default=False,
    #    help="Specify chapters to download separated by : (10:15).")
    parser.add_argument(
        "-ct", "--chapterthreads", default=1,
        help="Number of parallel chapters downloads.")
    parser.add_argument(
        "-wt", "--waittime", default=15,
        help="Wait time before retry if encountered with an error")
    parser.add_argument(
        "-rt", "--retries", default=10,
        help="Number of retries before giving up")

    args = parser.parse_args(['--location=' + location, '--folder=' + folder])

    return args


def about_story():

    source = os.path.join(basePath, arcRoot, folder, category, sfolder)
    cat = category.replace('_', ' ')
    cat_id = db.get_cat_id(cat)
    r = db.get_fanfic(cat_id, sfolder)

    _ = os.system("clear")

    if r["Abandoned"][0] == 1:
        print("WARNING: This story is ABANDONED and will NOT be updated.")
        print("")

    print("Title: ", r["Title"][0])
    print("Author: ", r["Author"][0])
    print("Folder Name: ", r["Folder"][0])
    print("Chapter Count: ", r["Count"][0])

    if r["Complete"][0] == 1:
        print("Complete: Yes")
    else:
        print("Complete: No")

    print("Last Updated: ", r["Update_Date"][0])
    print("Summary: ", textwrap.fill(r["Description"][0]))
    print("")

    cui.pause()

    menu_story()


def download_story():
    global basePath
    global folder
    global sfolder
    global category

    location = os.path.join(basePath, arcRoot, folder, category)
    source = os.path.join(location, sfolder)
    cat = category.replace('_', ' ')
    cat_id = db.get_cat_id(cat)
    fic = db.get_fanfic(cat_id, sfolder)

    _ = os.system("clear")
    
    abandon = fic['Abandoned'][0]
    complete = fic['Complete'][0]
    title = fic['Title'][0]
    author = fic['Author'][0]
    sfolder = fic['Folder'][0]
    chapters = fic['Count'][0]
    update = fic['Update_Date'][0]
    last = fic['Last_Checked'][0]
    fic_id = fic['Id'][0]

    if abandon == 1:
        print("WARNING: This story is ABANDONED and will NOT be updated.")
        print("")

    print("Title: ", title)
    print("Author: ", author)
    print("Folder Name: ", sfolder)
    print("Chapter Count: ", chapters)

    if complete == 1:
        print("Complete: Yes")
    else:
        print("Complete: No")

    print("Last Updated: ", update)
    print("")

    if (abandon == 1) or (complete == 1):
        cui.pause()
    else:
        ret = cui.menu_yesno('Download?', False)
        if ret == "Yes":
            chapter = input("Start Chapter [{0}]: ".format(chapters))
            if chapter == "":
                chapter = chapters
            else:
                chapter = int(chapter)
            process_story(location, fic_id, chapter)
            convert_story(source)
            cui.pause()
            menu_story()
        else:
            menu_story()

def process_story(location, fic_id, chapter):

    fic = db.get_fanfic_byId(fic_id)

    potential_keys = []

    title = fic['Title'][0]
    author = fic['Author'][0]
    folder = fic['Folder'][0]
    chapters = fic['Count'][0]
    update = fic['Update_Date'][0]
    last = fic['Last_Checked'][0]

    if fic['Internet'][0]:
        tmp = fic['Internet'][0]
        tmp2 = tmp.split('#')
        if len(tmp2) > 1:
            url = fic['Internet'][0].split('#')[1]
        else:
            url = tmp
    else:
        url = ''
        
    storyid = fic['StoryId'][0]
    fic_id = fic['Id'][0]

    url_check = current_fanfic.check_url(url)

    if url_check is True:

        ffargs = set_ffargs(location, folder)

        # Initialize class so we can retrieve actual story url
        fanfic = current_fanfic.fanfic(url, ffargs)
            
        # Get fanfic url
        url = fanfic.get_story_url(storyid)
            
        # Initialize class with correct url.
        fanfic = current_fanfic.fanfic(url, ffargs)
            
        # Download list of chapters
        fanfic.get_chapters()
        # Get current chapter count from site
        cc = fanfic.chapter_count

        if cc >= chapter:
            if cc == chapter:
                potential_keys = [chapter]
            else:
                potential_keys = [
                    i * 1 for i in range(chapter, (cc + 1))]
        
        # Show story about to be updated.
        update = fanfic.get_update_date()
                
        print('', end='\n')
        print(title, '/', folder, '/', update)
        
        # Set chapters to download
        fanfic.set_download_chapters(potential_keys)
        
        print('Downloading fanfic: ', title, '/', folder)

        # Download Story
        fanfic.download_fanfic()
        
        print('Downloaded fanfic: ', title, '/', folder)
        
        db.update_chapter_count(fic_id, cc)
        db.update_last_updated(fic_id, update)
        db.update_last_checked(fic_id, date.today())

        print('', end='\n')


def convert_story(location):

    files = get_files(location, "*.htm")

    for htm in files:
        convert_file(location, htm)


def convert_file(source, htm):

    txt = htm.split('.')[0] + '.txt'
    ifile = [os.path.abspath(os.path.join(source, htm))]
    ofile = os.path.abspath(os.path.join(source, txt))
    cmd = ['html2text', '--ignore-emphasis', '-b 80']
    msg = 'Converting file: {0} to text file: {1}'

    print(msg.format(htm, txt))

    with open(ofile, "w") as outfile:
        subprocess.call(cmd + ifile, stdout=outfile)

    for i in ifile:
        os.remove(i)


def toggle_complete():

    cat = category.replace('_', ' ')
    cat_id = db.get_cat_id(cat)
    fic_id = db.get_fic_id(cat_id, sfolder)

    db.set_complete(fic_id)

    menu_story()


def toggle_abandoned():

    cat = category.replace('_', ' ')
    cat_id = db.get_cat_id(cat)
    fic_id = db.get_fic_id(cat_id, sfolder)

    db.set_abandoned(fic_id)

    menu_story()


def get_entry(text, value):
    # Substitute {0} in text with value.
    return text.format(value)


def menu_editor():
    global editor

    menu = ['vi', 'gedit', 'geany']
    ret = cui.submenu(menu, "Choose Editor")

    editor = ret


def menu_options():
    menu = ['Path', 'Editor', 'Main Menu']
    ret = cui.submenu(menu, "Choose Option")

    if ret == "Editor":
        menu_editor()
    if ret == "Path":
        set_path()


def menu_story():

    menu = ['About Story', 'Download Story']
    menu = menu + ['Toggle Complete', 'Toggle Abandoned']
    menu = menu + ["Format File", "Main Menu"]
    ret = cui.submenu(menu, "Chose Story Option")

    if ret == "About Story":
        about_story()
    if ret == "Download Story":
        download_story()
    if ret == "Toggle Complete":
        toggle_complete()
    if ret == "Toggle Abandoned":
        toggle_abandoned()
    if ret == "Format File":
        if tfile:
            format_file(os.path.join(basePath, arcRoot,
                                     folder, category,
                                     sfolder, tfile))
        else:
            print("File must be selected first!")


def menu_sync():

    ret = subprocess.run(["which", "rclone"],
                         stdout=subprocess.PIPE,
                         stderr=subprocess.PIPE)
    check = ret.stdout.decode('utf-8').splitlines()[0]

    if check == '/usr/bin/rclone':

        if sync_server == '' or sync_path == '':
            sync = ["Config", "Main Menu"]
        else:
            sync = ["Config",
                    "Sync from Remote",
                    "Sync to Remote",
                    "Main Menu"]

        ret = cui.submenu(sync, "Choose Sync Option")

        if ret == "Config":
            setup_sync()
        if ret == "Sync from Remote":
            source = sync_server + ":" + sync_path + "/" + arcRoot
            dest = basePath + "/" + arcRoot
            run_sync(source, dest, sync_safe)
        if ret == "Sync to Remote":
            source = basePath + "/" + arcRoot
            dest = sync_server + ":" + sync_path + "/" + arcRoot
            run_sync(source, dest, sync_safe)
        if ret == "Main Menu":
            save_config()
            mainmenu()

    else:
        print("Command: rclone is required for sync to work!")
        cui.pause()


def setup_sync():
    global sync_server
    global sync_path
    global sync_safe

    while True:
        menu = {}
        menu[1] = get_entry("Server: {0}", sync_server)
        menu[2] = get_entry("Path: {0}", sync_path)
        menu[3] = get_entry("Safe Sync: {0}", sync_safe)
        menu[4] = "Previous Menu"

        options = menu.keys()
        _ = os.system("clear")
        print("Setup Sync Config")
        print('')

        for entry in options:
            opt = '%02d' % entry
            print(opt, menu[entry])
        print('')
        selection = input("Please Select: ")

        nodes = []

        if selection == "1":
            ret = subprocess.run(["rclone", "config", "dump"],
                                 stdout=subprocess.PIPE,
                                 stderr=subprocess.PIPE)
            cfg = ret.stdout.decode('utf-8')
            cfg2 = json.loads(cfg)
            for node in cfg2:
                nodes.append(node)

            print("")
            sync_server = cui.submenu(nodes, "Select Server")

        if selection == "2":
            if sync_server == "":
                print("Please specify server first!")
                cui.pause()
            else:
                sync_path = set_sync_path(sync_server)

        if selection == "3":
            if sync_path == "":
                print("Please specify path first!")
                cui.pause()
            else:
                if cui.menu_yesno("Do not overrite database?") == "Yes":
                    sync_safe = True
                else:
                    sync_safe = False

        if selection == "4":
            menu_sync()


def set_sync_path(server):

    tpath = ""

    while True:
        tpath = input("Enter root path to sync folder: ")
        ret = subprocess.run(["rclone", "lsf", "--dirs-only"] +
                             ["--max-depth", str(1)] +
                             [server + ":" + tpath],
                             stdout=subprocess.PIPE,
                             stderr=subprocess.PIPE)
        check = ret.stdout.decode('utf-8').splitlines()
        for folder in check:
            if arcRoot in folder:
                return tpath
        else:
            print("Invalid Folder!")
            cui.pause()


def run_sync(source, dest, update):

    cmd1 = ["rclone", "sync", "--progress", "--checksum"]

    # Prevent upload from overwriting htm folder or database as a safeguard.
    if update:
        cmd2 = ["--exclude", "htm/**", "--exclude", "FanfictionDB.db"]
        cmd = cmd1 + cmd2 + [source, dest]
    else:
        cmd = cmd1 + [source, dest]

    print(cmd)
    subprocess.call(cmd)
    cui.pause()

    menu_sync()


def choose_value(path, value):
    # Get sub-folders in path
    subdirs = next(os.walk(path))[1]
    subdirs.sort()

    if len(subdirs) > 0:
        ret = cui.submenu(subdirs, value)
    else:
        ret = ''
        print("No folders found!")
        cui.pause()

    return ret


def choose_file(path, value):
    global sfolder
    global tfile

    if not os.path.exists(path):
        print("No folder found!")
        ret = ''
        cui.pause()
    else:
        # get files in path
        lfiles1 = get_files(path, '*.txt')
        lfiles2 = get_files(path, '*.pdf')

        lfiles = lfiles1 + lfiles2

        ret = ''

        if len(lfiles) > 0:
            lfiles.append('*.txt')
            lfiles.append('*.pdf')
            lfiles.sort()
            ret = cui.submenu(lfiles, value)
        else:
            lfiles = get_files(path, '*.*')
            if len(lfiles) == 0:
                os.rmdir(path)

            sfolder = ''
            tfile = ''
            ret = tfile

    return ret


def get_files(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files, value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches


def get_folders(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for dirs in fnmatch.filter(files, value):
            matches.append(dirs)
    # Sort folder alphabetically
    matches.sort()

    return matches


def list_files(path, value):
    matches = get_files(path, value)
    print_list(matches)


def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches) / divisor)):
            j = divisor * (i * 1)
            result = ''
            for j in range(j, j + (divisor)):
                result = result + matches[j] + '\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor) * divisor)

    result = ''
    for m in range(l, len(matches)):
        result = result + matches[m] + '\t'
    print(result)


def run_script(value):
    subprocess.call(value, shell=True)
    cui.pause()


def format_file(value):

    print("Processing File... please wait.")

    t_file = value + ".tmp"

    f1 = open(value, "r")
    f2 = open(t_file, "w")

    fdata = f1.read()
    flines = fdata.splitlines()
    for line in flines:
        data1 = textwrap.fill(line, 80)
        data2 = data1.splitlines()
        for i in range(len(data2)):
            f2.write(data2[i] + "\n")
            if i == (len(data2) - 1):
                f2.write("\n")
    f2.flush()
    os.fsync(f2.fileno())

    f1.close
    f2.close

    os.remove(value)
    shutil.move(t_file, value)


def read_file(value):

    filename, file_extension = os.path.splitext(value)

    if file_extension == ".pdf":
        subprocess.call('xdg-open ' + value, shell=True)
    else:
        subprocess.call(editor + ' ' + value, shell=True)


def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)


def mov_files(path, dfolder, value):
    global tfile
    # Build destination path
    dpath = os.path.join(basePath, arcRoot, dfolder, category, sfolder)
    # Create destination folder if it doesn't exist.
    ensure_dir(dpath)

    list_files(path, value)
    confirm = input("Enter Y to proceed: ")
    if confirm.lower() == 'y':
        print("Archiving files")
        try:
            matches = get_files(path, value)
            for i in range(len(matches)):
                print("Archiving: ", matches[i])
                shutil.move(os.path.join(path, matches[i]),
                            os.path.join(dpath, matches[i]))
            print("Archive complete.")
        except Error as err:
            print("Problem Archiving File.")

    tfile = ''
    cui.pause()


def get_config():
    # create config dir if it doesn't exist and return path to config.
    home = os.path.expanduser("~")
    cfg = os.path.join(home, ".ff")
    ensure_dir(cfg)

    return os.path.join(cfg, "config.ini")


def load_config():
    global basePath
    global folder
    global dfolder
    global sfolder
    global category
    global tfile
    global editor
    global db_folder
    global db_name
    global sync_server
    global sync_path
    global sync_safe

    t_editor = ''
    t_path = ''
    t_dbname = ''
    t_dbfolder = ''
    t_sync_server = ''
    t_sync_path = ''
    t_sync_safe = '1'
    config = SafeConfigParser()

    cfg = get_config()

    if os.path.isfile(cfg):
        config.read(cfg)

        try:
            t_folder = config.get('archive', 'sourcefolder')
            t_category = config.get('archive', 'category')
            t_sfolder = config.get('archive', 'storyfolder')
            t_dfolder = config.get('archive', 'destfolder')
            t_tfile = config.get('archive', 'textfile')
            t_editor = config.get('options', 'editor')
            t_path = config.get('path', 'path')
            t_dbfolder = config.get('path', 'dbfolder')
            t_dbname = config.get('path', 'dbname')
            t_sync_server = config.get('sync', 'server')
            t_sync_path = config.get('sync', 'path')
            t_sync_safe = config.get('sync', 'safe')
        except:
            i = 0

        if t_sync_server:
            sync_server = t_sync_server

        if t_sync_path:
            sync_path = t_sync_path

        if t_editor:
            editor = t_editor

        if t_path:
            basePath = t_path

        if t_dbfolder:
            db_folder = t_dbfolder

        if t_dbname:
            db_name = t_dbname

        if t_sync_safe == '1':
            sync_safe = True
        else:
            sync_safe = False

        spath = os.path.join(basePath, arcRoot, t_folder)
        dpath = os.path.join(basePath, arcRoot, t_dfolder)

        # Validate parameters in config file are valid.
        if os.path.exists(dpath):
            dfolder = t_dfolder
        if os.path.exists(spath):
            folder = t_folder
            spath = os.path.join(spath, t_category)
            if os.path.exists(spath):
                category = t_category
                spath = os.path.join(spath, t_sfolder)
                if os.path.exists(spath):
                    sfolder = t_sfolder
                    sfile = os.path.join(spath, t_tfile)
                    if os.path.isfile(sfile):
                        tfile = t_tfile


def save_config():
    cfg = get_config()

    config = SafeConfigParser()

    config.read(cfg)

    if not config.has_section('archive'):
        config.add_section('archive')
    if not config.has_section('options'):
        config.add_section('options')
    if not config.has_section('path'):
        config.add_section('path')
    if not config.has_section('sync'):
        config.add_section('sync')

    config.set('archive', 'sourcefolder', folder)
    config.set('archive', 'category', category)
    config.set('archive', 'storyfolder', sfolder)
    config.set('archive', 'destfolder', dfolder)
    config.set('archive', 'textfile', tfile)
    config.set('options', 'editor', editor)
    config.set('path', 'path', basePath)
    config.set('path', 'rootfolder', arcRoot)
    config.set('path', 'dbfolder', db_folder)
    config.set('path', 'dbname', db_name)
    config.set('sync', 'server', sync_server)
    config.set('sync', 'path', sync_path)

    if sync_safe:
        config.set('sync', 'safe', '1')
    else:
        config.set('sync', 'safe', '0')

    with open(cfg, 'w') as f:
        config.write(f)


def set_path():
    global basePath

    msg = 'Sync Folder: {0}'
    print(msg.format(basePath))

    root = input("Enter root path to sync folder: ")

    if os.path.exists(root):
        basePath = root
    else:
        print("Invalid path, please try again.")
        cui.pause()
        _ = os.system("clear")
        set_path()


def mainmenu():
    global basePath
    global folder
    global dfolder
    global sfolder
    global category
    global tfile

    load_config()

    while True:
        menu = {}
        menu[1] = get_entry("Source Folder: {0}", folder)
        menu[2] = get_entry("Source Category: {0}", category)
        menu[3] = get_entry("Story: {0}", sfolder)
        menu[4] = get_entry("Dest Folder: {0}",
                            dfolder + "/" + category + "/" + sfolder)
        menu[5] = get_entry("Text File: {0}", tfile)
        menu[6] = "Read File"
        menu[7] = "Move File"
        menu[8] = "Story Options"
        menu[9] = "Sync Options"
        menu[10] = "Config Options"
        menu[11] = "Exit"

        options = menu.keys()
        _ = os.system("clear")
        print("Manage FanFiction Archive")
        print('')
        print(get_entry("Sync Folder: {0}", basePath))
        print('')
        for entry in options:
            opt = '%02d' % entry
            print(opt, menu[entry])
        print('')
        selection = input("Please Select: ")
        if selection == '1':
            root = os.path.join(basePath, arcRoot)
            if not os.path.exists(root):
                print("Please set a valid Sync Folder first!")
                basePath = ''
            else:
                folder = choose_value(os.path.join(basePath, arcRoot),
                                      "Choose Folder:")
        elif selection == '2':
            if not folder:
                print("Please choose a folder first!")
                cui.pause()
            else:
                category = choose_value(os.path.join(basePath, arcRoot,
                                        folder), "Choose Category:")
                sfolder = ''
                tfile = ''
        elif selection == '3':
            if not category:
                print("Please choose a category first!")
                cui.pause()
            else:
                sfolder = choose_value(os.path.join(basePath, arcRoot,
                                                    folder, category),
                                       "Choose Story Folder:")
                tfile = ''
        elif selection == '4':
            if not sfolder:
                print("Please choose a story first!")
                cui.pause()
            else:
                dfolder = choose_value(os.path.join(basePath, arcRoot),
                                       "Choose Destination Folder:")
        elif selection == '5':
            if not sfolder:
                print("Please choose a story first!")
                cui.pause()
            else:
                tfile = choose_file(os.path.join(basePath, arcRoot, folder,
                                                 category, sfolder),
                                    "Choose File:")
        elif selection == '6':
            if not tfile:
                print("Please select a file first!")
                cui.pause()
            else:
                read_file(os.path.join(basePath, arcRoot, folder,
                                       category, sfolder, tfile))
        elif selection == '7':
            if not tfile:
                print("Please select a file first!")
                cui.pause()
            else:
                mov_files(os.path.join(basePath, arcRoot, folder,
                                       category, sfolder), dfolder, tfile)
        elif selection == '8':
            if not sfolder:
                print("Please choose a story first!")
                cui.pause()
            else:
                menu_story()
        elif selection == '9':
            menu_sync()
        elif selection == '10':
            menu_options()
        elif selection == '11':
            save_config()
            sys.exit(0)
        else:
            print("Unknown Option Selected!")
            cui.pause()


def main():
    global db

    load_config()

    if not os.path.exists(basePath):
        set_path()
        save_config()
        ensure_dir(os.path.join(basePath, arcRoot))
        ensure_dir(os.path.join(basePath, arcRoot, db_folder))

    # Initialize DB
    db = FanficDB()
    # Call Main Menu
    mainmenu()


if __name__ == "__main__":
    sys.exit(main())



import os


def pause():
    # Wait for user input.
    _ = input("Press the <ENTER> key to continue...")


def get_entry(text, value):
    # Substitute {0} in text with value.
    return text.format(value)


def submenu_nav(values, prompt, idx, count, input_value=False, back=False):
    stop = len(values) - 1
    np = 0
    pp = 0
    mm = 0

    while True:
        print(prompt)
        print('')
        for i in range(0, len(values)):
            print(i, values[i])

        if idx < count:
            np = i + 1
            i = i + 1
            print(np, "Next Entry")
        if idx > 0:
            pp = i + 1
            i = i + 1
            print(pp, "Prev Entry")
        if back is True:
            if pp == 0:
                mm = np + 1
                print(mm, "Prev Menu")
            else:
                mm = pp + 1
                print(mm, "Prev Menu")
        print('')

        sel = int(input("Enter Selection: "))

        # Validate input is valid for current menu.
        if (sel >= 0) and (sel <= stop):
            if input_value:
                ret = sel
            else:
                ret = values[sel]
            break
        else:
            # Move 1 page ahead
            if sel == np:
                ret = "Next"
                break
            elif sel == pp:
                ret = "Prev"
                break
            elif sel == mm:
                ret = "Go Back"
                break
            else:
                print("Unknown Option Selected!")
                pause()
                _ = os.system("clear")

    return ret


def submenu(values, prompt, input_value=False):
    inc = 9
    start = 0
    stop = inc
    np = 0
    pp = 0

    while True:
        print(prompt)
        print('')
        for i in range(start, len(values)):
            print(i, values[i])
            j = i
            # limit number of entries to stop var.
            if i >= stop:
                # Stop offering next if the last entry is reached.
                if i < (len(values) - 1):
                    np = stop + 1
                    i = i + 1
                    print(np, "Next Page")
                    break
        # if stop > inc, allow prev page.
        if stop > inc:
            pp = i + 1
            i = i + 1
            print(pp, "Prev Page")
        print('')
        # parse input into number, blank is invalid.
        try:
            sel = int(input("Enter Selection: "))
        except:
            sel = j + 1
        # Validate input is valid for current menu.
        if (sel >= start) and (sel <= j):
            if input_value:
                ret = sel
            else:
                ret = values[sel]
            break
        else:
            # Move 1 page ahead
            if sel == np:
                start = start + 10
                stop = start + 9
                _ = os.system("clear")
            # Move 1 page back
            elif sel == pp:
                start = start - 10
                stop = start + 9
                _ = os.system("clear")
            elif sel < (len(values) - 10):
                start = sel
                stop = start + 9
                _ = os.system("clear")
            else:
                print("Unknown Option Selected!")
                pause()
                _ = os.system("clear")

    return ret


def menu_yesno(title='', clear=True):
    """ Prompt for yes no """

    while True:

        if clear:
            _ = os.system("clear")

        menu = ['Yes', 'No']
        ret = submenu(menu, title)

        if ret == 'Yes':
            return ret
        elif ret == 'No':
            return ret
        else:
            print("Invalid Option!")
            pause()


"""Define current fanfic class based on url."""

from fanfic_scraper.extractors.fanfictionnet import FanfictionNetFanfic
from fanfic_scraper.extractors.hpfanficarchive import HPFanficArchive

def check_url(url):
    ret = False

    if 'fanfiction.net' in url:
        return True
    if 'fanfictionproxy.net' in url:
        return True
    if 'hpfanficarchive.com' in url:
        return True

    return ret


def fanfic(fanfic_url, args):
    """Send the approriate class."""
    if 'fanfiction.net' in fanfic_url:
        return FanfictionNetFanfic(fanfic_url, args)
    if 'fanfictionproxy.net' in fanfic_url:
        return FanfictionNetFanfic(fanfic_url, args)
    if 'hpfanficarchive.com' in fanfic_url:
        return HPFanficArchive(fanfic_url, args)



#!/usr/bin/env python3
import argparse
import os
from urllib.parse import urlparse
from fanfic_scraper import current_fanfic


def main():
    """Parse input and download fanfic(s)."""
    parser = argparse.ArgumentParser(
        description=(
            'Downloads all fanfic chapters from'
            'the given url (currently works with fanfiction.net'
            '). Example - A url input '
            ' http://www.fanfiction.net/s/12861961/1/Magical-Player looks '
            'for the fanfic chapters in the url and downloads them all.'))

    parser.add_argument('urls', metavar='url', nargs='+',
                        help='fanfic urls to download')
    parser.add_argument(
        "-f", "--folder", required=True,
        help="Name of folder to place fanfic in.")
    parser.add_argument(
        "-l", "--location", default=os.getcwd(),
        help="set download location")
    parser.add_argument(
        "-c", "--chapters", default=False,
        help="Specify chapters to download separated by : (10:15).")
    parser.add_argument(
        "-ct", "--chapterthreads", default=5,
        help="Number of parallel chapters downloads.")
    parser.add_argument(
        "-wt", "--waittime", default=15,
        help="Wait time before retry if encountered with an error")
    parser.add_argument(
        "-rt", "--retries", default=10,
        help="Number of retries before giving up")

    args = parser.parse_args()

    for url in args.urls:
        # If https, check before using verify False
        urlscheme = urlparse(url)

        fanfic = current_fanfic.fanfic(url, args)
        fanfic.get_chapters()
        print('Downloading fanfic: ' + fanfic.name)

        # Get chapters to download
        if args.chapters:
            try:
                start_stop = args.chapters.split(':')
                if len(start_stop) == 1:
                    potential_keys = [int(start_stop[0])]
                elif len(start_stop) == 2:
                    potential_keys = [
                        i * 1 for i in range(int(start_stop[0]),
                                             int(start_stop[1]) + 1)]
                else:
                    raise SyntaxError(
                        "Chapter inputs should be separated by ':'")
            except TypeError:
                raise SyntaxError("Chapter inputs should be separated by ':'")

            fanfic.set_download_chapters(potential_keys)
        else:
            fanfic.set_download_chapters()

        # print(fanfic.chapter_count)

        fanfic.download_fanfic()
        print('Downloaded fanfic: ' + fanfic.title)


if __name__ == '__main__':
    main()



"""Base Fanfic class."""
import sys
import os
from collections import OrderedDict
import concurrent.futures
import requests
from requests.auth import HTTPBasicAuth
import urllib.parse


class BaseFanfic:
    """ Base Fanfic class. Contains chapters."""

    def __init__(self, fanfic_url, program_args):
        """Init function. Creates chapters for given fanfic."""
        self.title = ''
        self.url = fanfic_url
        self.name = program_args.folder
        self.location = program_args.location
        # Set download location
        self.download_location = os.path.abspath(
            os.path.join(self.location, self.name))
        # Set threads and retry values
        self.chapter_threads = program_args.chapterthreads
        self.wait_time = program_args.waittime
        self.max_retries = program_args.retries
        # Set verify mode
        self.verify_https = True
        self.all_chapters = None
        self.chapter_count = 0
        self.fanfic_id = 0

    def send_request(self, url):

        # https://github.com/TeamHG-Memex/aquarium

        proxy_url = 'http://127.0.0.1:8050/render.html?'
        proxy_url = proxy_url + 'proxy=tor&url={0}'.format(url)
        auth = HTTPBasicAuth('user', 'userpass')

        r = requests.get(proxy_url, auth=auth)

        return r

    def get_chapters(self):
        self.all_chapters = self.extract_chapters()
        self.chapter_count = len(self.all_chapters)

    def set_download_chapters(self, potential_keys=None, silent=False):
        """Set chapters to download."""
        if potential_keys:
            keys = list(set(potential_keys) & set(self.all_chapters.keys()))
        else:
            keys = list(self.all_chapters.keys())

        # Sort keys to make it ascending order and make it a new dict
        unsorted_chapters = {key: self.all_chapters[key]
                             for key in keys}

        self.chapters_to_download = OrderedDict(
            sorted(unsorted_chapters.items(), key=lambda t: t[0]))
        # Print downloading chapters
        if not silent:
            print("Downloading the below chapters:")
            print(sorted(keys))

    def story_info(self):

        self.get_chapters()
        potential_keys = [1]
        self.set_download_chapters(potential_keys, True)

        for chapter_num, chapter in self.chapters_to_download.items():
            ret = chapter.story_info()
            break

        return ret

    def download_fanfic(self):
        """Begin download of chapters in the fanfic."""

        if not os.path.exists(self.download_location):
            os.makedirs(self.download_location)

        with concurrent.futures.ThreadPoolExecutor(
                max_workers=self.chapter_threads) as executor:
            future_to_chapter = {
                executor.submit(chapter.download_chapter): chapter_num
                for chapter_num, chapter in self.chapters_to_download.items()}

            for future in concurrent.futures.as_completed(future_to_chapter):
                chapter_num = future_to_chapter[future]
                try:
                    future.result()
                except Exception as exc:
                    print('Chapter-%g generated an exception: %s'
                          % (chapter_num, exc))
                    import traceback
                    exc_info = sys.exc_info()
                    traceback.print_exception(*exc_info)
                else:
                    print('Downloaded: Chapter-%g' % (chapter_num))

    def extract_chapters(self):
        """Extract chapters function (backbone)."""
        pass


class BaseChapter:
    """Base Chapter class."""

    def __init__(self, fanfic, chapter_num, chapter_url):
        """Initialize constants required for download."""
        # Extract necessary information from the fanfic object
        self.fanfic_url = fanfic.url
        self.fanfic_id = fanfic.fanfic_id
        self.fanfic_name = fanfic.name
        self.fanfic_download_location = fanfic.download_location
        # Create chapter specific variables
        self.chapter_num = chapter_num
        self.chapter_url = chapter_url
        # Threads and retry time
        self.wait_time = fanfic.wait_time
        self.max_retries = fanfic.max_retries
        # Set verify mode
        self.verify_https = fanfic.verify_https
        # Get download chapter location
        self.chapter_location = fanfic.download_location

    def send_request(self, url):

        # https://github.com/TeamHG-Memex/aquarium

        url = urllib.parse.quote_plus(url)

        proxy_url = 'http://127.0.0.1:8050/render.html?'
        proxy_url = proxy_url + 'proxy=tor&url={0}'.format(url)
        auth = HTTPBasicAuth('user', 'userpass')

        r = requests.get(proxy_url, auth=auth)

        return r

    def download_chapter(self):
        """Download chapter (backbone)."""
        pass

    def story_info(self):
        pass



# -*- coding: utf-8 -*-
from distutils.core import setup

packages = \
['fanfic_scraper', 'fanfic_scraper.extractors']

package_data = \
{'': ['*']}

install_requires = \
['beautifulsoup4>=4.7,<5.0',
 'fake_useragent>=0.1.11,<0.2.0',
 'html2text>=2018.1,<2019.0',
 'html5lib>=1.0,<2.0',
 'pony>=0.7.10,<0.8.0',
 'pysocks>=1.7,<2.0',
 'requests>=2.22,<3.0',
 'tldextract>=2.2,<3.0']

entry_points = \
{'console_scripts': ['fanfic-scraper = fanfic_scraper.fanfic_scraper:main',
                     'ff = fanfic_scraper.ff:main',
                     'ff-fif = fanfic_scraper.ff_fif:main',
                     'ff-h2t = fanfic_scraper.ff_h2t:main',
                     'ff-rf = fanfic_scraper.ff_rf:main',
                     'ffdl = fanfic_scraper.ffdl:main']}

setup_kwargs = {
    'name': 'fanfic-scraper',
    'version': '0.7.4',
    'description': 'Scrapes fanfics, maintains download db, organizes folders when reading.',
    'long_description': None,
    'author': 'Kevin Meredith',
    'author_email': 'kevin@meredithkm.info',
    'url': None,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.5,<4.0',
}


setup(**setup_kwargs)



#!/usr/bin/python3

import os
import sys
import argparse
import shutil
import fnmatch
import re


def pause():
    # Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")


def print_format(text, value):
    # Substitute {0} in text with value.
    print(text.format(value))


def get_files(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files, value):
            f = root + '/' + filename
            matches.append(f)

    # Sort files alphabetically
    matches.sort()

    return matches


def ensure_dir(path):
    if not os.path.exists(path):
        os.makedirs(path)


def list_files(path, value):
    matches = get_files(path, value)
    print_list(matches)


def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches) / divisor)):
            j = divisor * (i * 1)
            result = ''
            for j in range(j, j + (divisor)):
                result = result + matches[j] + '\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor) * divisor)

    result = ''
    for m in range(l, len(matches)):
        result = result + matches[m] + '\t'
    print(result)


def get_prefix(file):
    matches = []
    t = ''

    regex1 = "([a-zA-Z]+[0-9]+[a-zA-Z]+-)"
    regex2 = "([a-zA-Z]+[0-9]+-)"
    regex3 = "([a-zA-Z]+-)"
    regex4 = "([a-zA-Z]+)"
    regex5 = [regex1, regex2, regex3, regex4]

    r = re.compile('|'.join(regex5))

    m = r.match(file)

    if m:
        if (m.group(1)):
            t = m.group(1)
        elif (m.group(2)):
            t = m.group(2)
        elif (m.group(3)):
            t = m.group(3)
        elif (m.group(4)):
            t = m.group(4)

    return t


def main():

    """Rename fanfic chapters."""
    parser = argparse.ArgumentParser(
        description=('Utility to rename fanfic chapters'))

    parser.add_argument('folder', help='fanfic folder to process')

    args = parser.parse_args()

    source = args.folder

    txt = get_files(source, "*.txt")
    htm = get_files(source, "*.htm")

    files = txt + htm

    abort = 0

    for f in files:
        d = os.path.dirname(f)
        f = os.path.basename(f)
        cat = os.path.basename(d)

        if '_' in f:
            target = f.replace('_', '-')
            ren_file(d, f, target)
            f = target

        p = get_prefix(f)

        if p == '':
            abort = 1
        else:
            try:
                n = int(f.replace(p, '').split('.')[0])
            except:
                abort = 1

        if (abort == 0):
            p = p.split('-')[0] + '-'
            num = '%03d' % n
            ext = f.replace(p, '').split('.')[1]
            target = p + num + '.' + ext
            ren_file(d, f, target)
        else:
            abort = 0


def ren_file(folder, orig, new):
    source = os.path.join(folder, orig)
    target = os.path.join(folder, new)

    print("Fanfic: " + folder)
    print("Renaming: " + orig + " to " + new)

    shutil.move(source, target)


if __name__ == "__main__":
    sys.exit(main())



#!/usr/bin/python3

import os
import sys
import fnmatch
import subprocess
import argparse


def pause():
    # Wait for user input.
    programPause = input("Press the <ENTER> key to continue...")


def print_format(text, value):
    # Substitute {0} in text with value.
    print(text.format(value))


def get_files(path, value):
    matches = []
    for root, dirs, files in os.walk(path):
        for filename in fnmatch.filter(files, value):
            matches.append(filename)
    # Sort files alphabetically
    matches.sort()

    return matches


def print_list(matches):
    divisor = 6
    result = ''

    for i in range(int(len(matches) / divisor)):
            j = divisor * (i * 1)
            result = ''
            for j in range(j, j + (divisor)):
                result = result + matches[j] + '\t'
            print(result)

    k = len(matches) % divisor
    l = (int(len(matches) / divisor) * divisor)

    result = ''
    for m in range(l, len(matches)):
        result = result + matches[m] + '\t'
    print(result)


def convert_file(source, htm):

    txt = htm.split('.')[0] + '.txt'
    ifile = [os.path.abspath(os.path.join(source, htm))]
    ofile = os.path.abspath(os.path.join(source, txt))
    cmd = ['html2text', '--ignore-emphasis', '-b 80']
    msg = 'Converting file: {0} to text file: {1}'

    print(msg.format(htm, txt))

    with open(ofile, "w") as outfile:
        subprocess.call(cmd + ifile, stdout=outfile)

    for i in ifile:
        os.remove(i)


def main():

    """Rename fanfic chapters."""
    parser = argparse.ArgumentParser(
        description=('Utility to conver html fanfic chapters to text'))

    parser.add_argument('folder', help='fanfic folder to process')

    args = parser.parse_args()

    source = args.folder

    if not os.path.exists(source):
        sys.stderr.write("ERROR: Folder %r was not found!" % (source,))
        return 1

    files = get_files(source, "*.htm")

    # print_list(files)

    source = os.path.realpath(source)

    for htm in files:
        convert_file(source, htm)


if __name__ == "__main__":
    sys.exit(main())
